#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>

#include <sys/socket.h>
#include <sys/ioctl.h>

#ifdef __linux__
# include <linux/in.h> 
# include <linux/if.h>
#endif

#ifdef IFSTATS_USE_LINUX_KERNEL_PPP
# include <fcntl.h>
# include <netdb.h>

# ifdef __linux__
#  include <linux/ppp_defs.h>
#  include <linux/if_ppp.h>
# endif
#endif

#include "../include/string.h"
#include "../include/os.h"

#include "ifstats.h"


static void GetMilliTime(long *t);

int IFSIsUp(if_stats_desc_struct *ifsd);
int IFSGetIP(if_stats_desc_struct *ifsd);
int IFSGetBandwidth(if_stats_desc_struct *ifsd);

int IFSInit(if_stats_desc_struct *ifsd, const char *interface);
int IFSFetch(if_stats_desc_struct *ifsd);
void IFSShutdown(if_stats_desc_struct *ifsd);


/*
 *	Gets time of day in milliseconds since midnight.
 */
static void GetMilliTime(long *t)
{
        struct timeval tv[1];
        
        if(gettimeofday(tv, NULL) < 0)
            return;

        *t = (tv->tv_sec % 86400) * 1000 + tv->tv_usec / 1000;
}


/*
 *	Checks if interface is up, returns true if it is.
 */
int IFSIsUp(if_stats_desc_struct *ifsd)
{
#ifdef __linux__
/* Linux does not store the interface state in any of the /proc files so
 * the state needs to be obtained using a socket and to probe the
 * interface for the correct state.
 *
 * The structures ifreq and in_addr are defined by the linux header files
 * so this section is not portable.
 */
        int s, status;
	const char *interface_name;
        struct ifreq ifr;  


	if(ifsd == NULL)
	    return(0);

	interface_name = (const char *)ifsd->interface;
        if(interface_name == NULL)
            return(0);

        /* Create a UDP socket. */
        s = socket(AF_INET, SOCK_DGRAM, 0);
        if(s < 0)
            return(0);

        /* Set interface name explicitly from ifsd. */
        strncpy(
            ifr.ifr_name,
            interface_name,
            IFNAMSIZ
        );
        ifr.ifr_name[IFNAMSIZ - 1] = '\0';

        /*   Get flags from interface and put into interface request
	 *   structure.
	 */
        status = ioctl(s, SIOCGIFFLAGS, (caddr_t)&ifr);

	/* Close socket. */
	close(s);
	s = -1;

	/* ioctl() failed? */
	if(status < 0)
	    return(0);

	return(ifr.ifr_flags & IFF_UP);
#endif	/* __linux__ */
}


/*
 *	Gets IP address of interface and stores it on the given ifsd
 *	structure.
 */
int IFSGetIP(if_stats_desc_struct *ifsd)
{
#ifdef __linux__
/* Linux does not store the interface address in any of the /proc files so
 * the address needs to be obtained using a socket and to probe the 
 * interface for the correct address.
 *
 * The structures ifreq and in_addr are defined by the linux header files
 * so this section is not portable.
 */
        int s, status;
	const char *interface_name;
        struct ifreq ifr;
        struct in_addr ina;
	if_ip_addr_union ip;


        if(ifsd == NULL)
            return(-1);

        /* Must have a specified interface. */
	interface_name = (const char *)ifsd->interface;
        if(interface_name == NULL)
            return(-1);

        /* Create a UDP socket. */
        s = socket(AF_INET, SOCK_DGRAM, 0);
        if(s < 0)
            return(-1);

        /* Set interface name explicitly from ifsd. */
        strncpy(
            ifr.ifr_name,
            interface_name,
            IFNAMSIZ
        );
        ifr.ifr_name[IFNAMSIZ - 1] = '\0';

        /*   Get address of interface and put into interface request
         *   structure.
         */
        status = ioctl(s, SIOCGIFADDR, &ifr);

	/* Close socket. */
	close(s);
	s = -1;

	/* ioctl() failed? */
        if(status < 0)
            return(-1);

        /* Give struct in_addr ina the address of the device. */
        ina = ((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr;
        ip.whole = ina.s_addr;		/* It's in network byte order. */

	/* Set IP address of interface on ifsd. */
	ifsd->ip.whole = ip.whole;

	return(0);
#else	/* __linux__ */
/* Add support for fetching of interface addresses for non-linuxes here.
 */



	return(0);
#endif
}

/*      
 *	Gets the bandwidth of the interface.
 */
int IFSGetBandwidth(if_stats_desc_struct *ifsd)
{
#ifdef IFSTATS_USE_LINUX_KERNEL_PPP
#ifdef HAS_SIOCGIFINDEX
        int s, status;
        struct ifreq ifr;
        struct in_addr ina;
        if_ip_addr_union ip;


        if(ifsd == NULL)
            return(-1);

        /* Must have a specified interface. */
        if(ifsd->interface == NULL)
            return(-1);


        /* Create a UDP socket. */
        s = socket(AF_INET, SOCK_DGRAM, 0);
        if(s < 0)
            return(-1);

        /* Set interface name explicitly from ifsd. */
        strncpy(
            ifr.ifr_name,
            ifsd->interface,
            IFNAMSIZ  
        );
        ifr.ifr_name[IFNAMSIZ - 1] = '\0';
        
        /*   Get bandwidth of interface and put into interface request
         *   structure.
         */
        status = ioctl(s, SIOCGIFINDEX, &ifr);

        /* Close socket. */
        close(s);
	s = -1;

        /* ioctl() failed? */
        if(status < 0)
            return(-1);

	ifsd->bandwidth = ifr.ifr_bandwidth;
/*
printf("Bandwidth: %i\n", ifsd->bandwidth);
 */
#endif	/* HAS_SIOCGIFINDEX */

        return(0);
#else




	return(0);
#endif	/* IFSTATS_USE_LINUX_KERNEL_PPP */
}


/*
 *	Connect to interface and stores resource information
 *	about connection in ifsd.
 */
int IFSInit(if_stats_desc_struct *ifsd, const char *interface)
{
#ifdef IFSTATS_USE_LINUX_KERNEL_PPP
	char *service = "ifmon";
	char *protocol = "tcp";
        struct servent *sp;
        struct sockaddr_in addr[1];
        int s, p;


	if(ifsd == NULL)
	    return(-1);

	/* Set to default interface name as needed. */
	if(interface == NULL)
	    interface = DEF_INTERFACE_NAME;


	/* Reset values. */
        ifsd->socket = -1;
        ifsd->interface = NULL;


	/* Get service information. */
        sp = getservbyname(service, protocol);
        if(sp == NULL)   
        {
	    fprintf(
		stderr,
 "%s: Cannot find %s in /etc/services\n",
		"getservbyname()",
		service
	    );
            fprintf(
                stderr,
"Make sure you have a line in /etc/services for interface monitoring, ie:\n\
ifmon           6020/tcp                # Interface stats monitoring.\n"
            );
            return(-1);
        }
        else
        {
            p = sp->s_port;
        }

	/* Create a TCP socket. */
        s = socket(AF_INET, SOCK_STREAM, 0);
        if(s < 0)
        {
            perror("socket");
            return(-1);
        }

        addr->sin_family = AF_INET;
        memset(&addr->sin_addr, 0, sizeof(addr->sin_addr));
        addr->sin_port = p;


        /* Bind the socket. */
        if(bind(s, (struct sockaddr *)addr, sizeof(addr)))
        {
            perror("bind");
            shutdown(s, SHUT_RDWR);
            return(-1);
        }

        if(listen(s, 5) < 0)
        {
            perror("listen");
            shutdown(s, SHUT_RDWR);    
            return(-1);
        }


	/* Set up descriptor information in ifsd. */
	ifsd->disgarded_frames = 0;
        ifsd->rx_bytes = 0;
        ifsd->sx_bytes = 0;
        ifsd->rex_bytes = 0;
        ifsd->sex_bytes = 0;
        ifsd->rx_packets = 0;
        ifsd->sx_packets = 0;
        ifsd->rx_lrqs = 0;
        ifsd->sx_lrqs = 0;

	ifsd->ip.whole = 0;

	GetMilliTime(&ifsd->time);

	ifsd->socket = s;
	ifsd->interface = strdup(interface);


        /* Clear interface request struct. */
        memset(&ifsd->ifr, 0, sizeof(struct ifreq));

        /* More interface setups... */
        strncpy(
            ifsd->ifr.ifr_ifrn.ifrn_name,
            interface,				/* Interface name */
            IFNAMSIZ
        );
	ifsd->ifr.ifr_ifrn.ifrn_name[IFNAMSIZ - 1] = '\0';

        ifsd->ifr.ifr_ifru.ifru_data = (caddr_t)ifsd->ppp_info;
        ifsd->ppp_st_ptr = &ifsd->ppp_info->p;
        memset(ifsd->last_ppp_st, 0x00, sizeof(struct pppstat));

        return(0);
#else
        if(ifsd == NULL)
            return(-1);

        /* Set to default interface name as needed. */
        if(interface == NULL)
            interface = DEF_INTERFACE_NAME;


        /* Reset values. */
        ifsd->interface = NULL;
	ifsd->stats_file = NULL;

        /* Set up descriptor information in ifsd. */
        ifsd->disgarded_frames = 0;
        ifsd->rx_bytes = 0;
        ifsd->sx_bytes = 0;
        ifsd->rex_bytes = 0;
        ifsd->sex_bytes = 0;
        ifsd->rx_packets = 0;
        ifsd->sx_packets = 0;
        ifsd->rx_lrqs = 0;
        ifsd->sx_lrqs = 0;

        ifsd->ip.whole = 0;

        GetMilliTime(&ifsd->time);


	/* Set up interface name and file names. */
        ifsd->interface = strdup(interface);

        ifsd->stats_file_is_virtual = 1;	/* Interace stats file is
						 * virtual since it is under
						 * /proc/<*>.
						 */
	ifsd->stats_file = strdup("/proc/net/dev");

	return(0);
#endif	/* IFSTATS_USE_LINUX_KERNEL_PPP */
}

/*
 *	Fetches new interface stats and updates the given ifsd structure
 *	with them.
 *
 *	Returns non-zero on failure.
 */
int IFSFetch(if_stats_desc_struct *ifsd)
{
#ifdef IFSTATS_USE_LINUX_KERNEL_PPP
	int s;
	struct pppstat *ppp_st_ptr;


        if(ifsd == NULL)
            return(-1);

	/* Get socket to interface. */
	s = ifsd->socket;
	if(s < 0)
	    return(-1);

	/*   Fetch ppp stats from ppp interface and put into
         *   interface request structure.
	 */
	ioctl(s, SIOCGPPPSTATS, (caddr_t)&ifsd->ifr);
	ppp_st_ptr = ifsd->ppp_st_ptr;


	/* Set newly fetched stats. */
	ifsd->disgarded_frames = ppp_st_ptr->ppp_discards;
        ifsd->rx_bytes = ppp_st_ptr->ppp_ibytes;
        ifsd->sx_bytes = ppp_st_ptr->ppp_obytes;
        ifsd->rex_bytes = ppp_st_ptr->ppp_ierrors;
        ifsd->sex_bytes = ppp_st_ptr->ppp_oerrors;
        ifsd->rx_packets = ppp_st_ptr->ppp_ipackets;
        ifsd->sx_packets = ppp_st_ptr->ppp_opackets;
        ifsd->rx_lrqs = ppp_st_ptr->ppp_ilqrs;
        ifsd->sx_lrqs = ppp_st_ptr->ppp_olqrs;

	GetMilliTime(&ifsd->time);

	return(0);
#else
	int matched_interface = 0;
	const char *interface_name, *stats_file;
	FILE *fp;
#define buf_len		1024
	char buf[buf_len];
	int line_num = 0;


        if(ifsd == NULL)
            return(-1);

	/* Get interface name. */
	interface_name = (const char *)ifsd->interface;
	if(interface_name == NULL)
	    return(-1);

        /* Get interface stats file and open it. */
        stats_file = (const char *)ifsd->stats_file;
        if(stats_file == NULL)
            return(-1);
        fp = fopen(stats_file, "rb");
        if(fp == NULL)
            return(-1);

	/* Begin reading and parsing. */
	while(fgets(buf, buf_len, fp) != NULL)
	{
	    /* Begin parsing when we get to line number index 2. */
	    if(line_num >= 2)
	    {
		/* From line 2 and so on the format is expected to be as
		 * follows (17 values):
		 * <ifname>: <r_bytes> <r_packets> <r_errs> <r_drop>
		 * <r_fifo> <r_frame> <r_compressed> <r_multicast>
		 * <s_bytes> <s_packets> <s_errs> <s_drop> <s_fifo>
		 * <s_colls> <s_carrier> <s_compressed>
		 */
		char cur_ifname[256];
		char *strptr;

		unsigned long	r_bytes, r_packets, r_errs, r_drop,
				r_fifo, r_frame, r_compressed, r_multicast,
				s_bytes, s_packets, s_errs, s_drop,
				s_fifo, s_colls, s_carrier, s_compressed;

		/* Get interface name of current line. */
		strptr = buf;
		while(ISBLANK(*strptr))
		    strptr++;
		strncpy(cur_ifname, strptr, 256);
		cur_ifname[256 - 1] = '\0';
		strptr = strchr(cur_ifname, ':');
		if(strptr != NULL)
		    (*strptr) = '\0';

		/* Compare interface names. */
		if(!strcmp(interface_name, cur_ifname))
		{
		    /* Interface names match, get stats from this line
		     * pointed to by strptr.
		     */
		    strptr = strchr(buf, ':');
		    if(strptr != NULL)
			strptr++;
		    else
			strptr = buf;
		    while(ISBLANK(*strptr))
			strptr++;

		    /* Now strptr is positioned at the start of the first
		     * value.
		     */
		    sscanf(
			strptr,
"%lu %lu %lu %lu\
 %lu %lu %lu %lu\
 %lu %lu %lu %lu\
 %lu %lu %lu %lu",
			&r_bytes, &r_packets, &r_errs, &r_drop,
			&r_fifo, &r_frame, &r_compressed, &r_multicast,
			&s_bytes, &s_packets, &s_errs, &s_drop,
			&s_fifo, &s_colls, &s_carrier, &s_compressed
		    );

		    ifsd->disgarded_frames = 0;

		    ifsd->rx_bytes = r_bytes;
		    ifsd->sx_bytes = s_bytes;
		    ifsd->rex_bytes = r_errs;
		    ifsd->sex_bytes = s_errs;
		    ifsd->rx_packets = r_packets;
		    ifsd->sx_packets = s_packets;
		    ifsd->rx_lrqs = 0;
		    ifsd->sx_lrqs = 0;

		    /* Update time to reflect time of newly fetched
		     * stats.
		     */
		    GetMilliTime(&ifsd->time);

		    /* Mark that we matched our interface from the data
		     * read from file.
		     */
		    matched_interface = 1;

		    /* Break, don't bother reading subsequent lines. */
		    break;
		}
	    }

	    /* Increment current line number count to next line. */
	    line_num++;
	}

        /* Close interface stats file. */
        fclose(fp);
        fp = NULL;

#undef buf_len

	if(matched_interface)
	    return(0);
	else
	    return(-1);
#endif	/* IFSTATS_USE_LINUX_KERNEL_PPP */
}


/*
 *	Shuts down all resources specified on the given ifsd and
 *	deallocates/resets all values on the ifsd.
 *
 *	Does not deallocate the ifsd structure itself.
 */
void IFSShutdown(if_stats_desc_struct *ifsd)
{
#ifdef IFSTATS_USE_LINUX_KERNEL_PPP
        if(ifsd == NULL)
            return;

	/* Close listening socket. */
	if(ifsd->socket > -1)
	{
	    shutdown(ifsd->socket, SHUT_RDWR);
	    ifsd->socket = -1;
	}

	/* Interface name. */
	free(ifsd->interface);
	ifsd->interface = NULL;
#else
        if(ifsd == NULL)
            return;

        /* Interface name. */
        free(ifsd->interface);
        ifsd->interface = NULL;

	ifsd->stats_file_is_virtual = 0;

	/* Interface stats file. */
	free(ifsd->stats_file);
	ifsd->stats_file = NULL;

#endif	/* IFSTATS_USE_LINUX_KERNEL_PPP */


        /* Reset values. */
        ifsd->disgarded_frames = 0;
        ifsd->rx_bytes = 0;
        ifsd->sx_bytes = 0;
        ifsd->rex_bytes = 0;
        ifsd->sex_bytes = 0;
        ifsd->rx_packets = 0;
        ifsd->sx_packets = 0;
        ifsd->rx_lrqs = 0;
        ifsd->sx_lrqs = 0;

        ifsd->ip.whole = 0;

        ifsd->time = 0;
}
