/*
              Interface Monitor Window: GUI Management

	Functions:

	int IMWInit(ifmon_win_struct *imw, int argc, char *argv[])
	void IMWDraw(ifmon_win_struct *imw)
	int IMWManage(ifmon_win_struct *imw, event_t *event)
	void IMWMap(ifmon_win_struct *imw)
	void IMWUnmap(ifmon_win_struct *imw)
	void IMWDestroy(ifmon_win_struct *imw)

	---

 */

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <unistd.h>

#include "../include/disk.h"
#include "../include/string.h"

#include "../include/osw-x.h"
#include "../include/widget.h"

#include "blitting.h"

#include "xifmon.h"


/*
 *	Hint messages:
 */
#define CONNECT_BTN_HINT_MESG		"Connects interface"
#define DISCONNECT_BTN_HINT_MESG	"Disconnects interface"
#define STATS_BTN_HINT_MESG		"Display interface statistics"
#define EXIT_BTN_HINT_MESG		"Exits the program"


/*
 *	Initialize interface monitor window.
 */
int IMWInit(ifmon_win_struct *imw, int argc, char *argv[])
{
	int i, x, y;
	unsigned int width, height;
	image_t *image;
	pixmap_t pixmap_mask;
	struct stat stat_buf;
	const char *arg;
	const char *interface = DEF_INTERFACE_NAME;
	int bandwidth_units = IF_BANDWIDTH_UNITS_BYTES_PER_SECOND;
	int bandwidth = DEF_BANDWIDTH;


	if(imw == NULL)
	    return(-1);


        imw->connect_script = NULL;
        imw->disconnect_script = NULL;
	imw->interface = strdup(interface);


	/* Parse arguments. */
        for(i = 1; i < argc; i++)
        {
	    arg = (const char *)argv[i];
            if(arg == NULL)
                continue;


	    /* First non-command argument (without the '-' prefix) is
	     * assumed to be the interface name.
	     */
	    if((i == 1) && ((*arg) != '-') && ((*arg) != '+'))
	    {
		interface = (const char *)arg;

		free(imw->interface);
		imw->interface = strdup(interface);
	    }
            /* Interface bandwidth in bits per second. */
            else if(strcasepfx(arg, "--bandwidth_bits") ||
                    strcasepfx(arg, "-bandwidth_bits")
            )
            {
                i++;
                if(i < argc)
                {
                    arg = (const char *)argv[i];

                    /* Get bandwidth value from argument (in bits per
                     * second). Store in bits per second.
                     */
                    bandwidth = atoi(arg);

                    /* Mark that the bandwidth units must be displayed
                     * in bits per second.
                     */
                    bandwidth_units = IF_BANDWIDTH_UNITS_BITS_PER_SECOND;

                    /* Check if bandwidth value is too low. */
                    if(bandwidth < BANDWIDTH_MIN)
                    {
                        fprintf(
                            stderr,
"Warning: %s: Value cannot be less than %i.\n",
                            argv[i - 1],
                            BANDWIDTH_MIN
                        );

                        bandwidth = BANDWIDTH_MIN;
                    }
                }
            }
	    /* Interface bandwidth in bytes per second. */
	    else if(strcasepfx(arg, "--b") ||
                    strcasepfx(arg, "-b")
	    )
	    {
		i++;
		if(i < argc)
		{
                    arg = (const char *)argv[i];

		    /* Get bandwidth value from argument (in bytes per
		     * second). Store in bits per second.
		     */
		    bandwidth = atoi(arg) * 8;

		    /* Mark that the bandwidth units must be displayed
		     * in bytes per second.
		     */
		    bandwidth_units = IF_BANDWIDTH_UNITS_BYTES_PER_SECOND;

		    /* Check if bandwidth value is too low. */
		    if(bandwidth < BANDWIDTH_MIN)
		    {
			fprintf(
			    stderr,
"Warning: %s: Value cannot be less than %i.\n",
			    argv[i - 1],
			    BANDWIDTH_MIN
			);

			bandwidth = BANDWIDTH_MIN;
		    }
		}
	    }
            /* Initialization (connect) script: --initialize */
            else if(strcasepfx(arg, "--i") ||
                    strcasepfx(arg, "-i") ||
                    strcasepfx(arg, "--c") ||
                    strcasepfx(arg, "-c")
            )
            {
		i++;
		if(i < argc)
		{
                    arg = (const char *)argv[i];

		    free(imw->connect_script);
		    imw->connect_script = strdup(arg);

		    if(stat(arg, &stat_buf))
			fprintf(
			    stderr,
			    "%s: No such file.\n",
			    arg
			);
		}
	    }
            /* Shutdown (disconnect) script: --shutdown */
            else if(strcasepfx(arg, "--s") ||
                    strcasepfx(arg, "-s") ||
                    strcasepfx(arg, "--d") ||
                    strcasepfx(arg, "-d")
            )
            {
                i++;
                if(i < argc)
                {
		    arg = (const char *)argv[i];

                    free(imw->disconnect_script);
                    imw->disconnect_script = strdup(arg);

                    if(stat(arg, &stat_buf))
                        fprintf(
                            stderr,
                            "%s: No such file.\n",
                            arg
                        );
                }
            }
	}


	/* Set default values. */
	image = xifmon_image.bkg;
	if(image == NULL)
	{
	    width = 320;
	    height = 240;
	}
	else
	{
	    width = image->width;
	    height = image->height;
	}

	if(osw_gui[0].def_geometry_set)
	{
	    x = osw_gui[0].def_toplevel_x;
            y = osw_gui[0].def_toplevel_y;
	}
	else
	{
	    x = 0;
	    y = 0;
	}

	imw->map_state = 0;
	imw->is_in_focus = 0;
	imw->x = x;
	imw->y = y;
	imw->width = width;
	imw->height = height;

	imw->in_drag = False;

	imw->if_state = IF_STATE_OFF;
	imw->bandwidth_units = bandwidth_units;
	imw->bandwidth = bandwidth;


        /* Create toplevel. */
        if(
            OSWCreateWindow(
                &imw->toplevel,
                osw_gui[0].root_win,
                imw->x, imw->y,
                imw->width, imw->height
            )
        )
            return(-1);

	imw->toplevel_buf = 0;
	imw->toplevel_img = NULL;

        OSWSetWindowWMProperties(
            imw->toplevel,
            PROG_NAME,		/* Title. */
            PROG_NAME,		/* Icon title. */
            xifmon_image.icon_pm,	/* Icon. */
            !osw_gui[0].def_geometry_set,       /* Let WM set coordinates? */
            imw->x, imw->y,
            imw->width, imw->height,
            imw->width, imw->height,
            WindowFrameStyleNaked,
            NULL, 0
        ); 
        OSWSetWindowInput(
            imw->toplevel,
            OSW_EVENTMASK_TOPLEVEL | ButtonPressMask | ButtonReleaseMask |
            PointerMotionMask | ExposureMask
        );

	/* Shape window. */
	pixmap_mask = WidgetPixmapMaskFromImage(xifmon_image.bkg);
	if(pixmap_mask != 0)
	{
#ifdef X_H
            XShapeCombineMask(
		osw_gui[0].display,
                imw->toplevel,
                ShapeBounding,
                0, 0,
                pixmap_mask,
                ShapeSet
            );
#endif	/* X_H */
	}


	/* Initialize buttons. */
	x = 35;
	y = 8;
        if(
            PBtnInit(
                &imw->connect_btn,
                imw->toplevel,
                x, y,
                16, 16,
                NULL,
                PBTN_TALIGN_CENTER,
                xifmon_image.connect_label,
                imw,
                IMWConnectPBCB
            )
        )
            return(-1);
	PBtnSetHintMessage(
	    &imw->connect_btn,
	    CONNECT_BTN_HINT_MESG
	);

	x = 54;
	y = 8;
        if(
            PBtnInit(
                &imw->disconnect_btn,
                imw->toplevel, 
                x, y,
                16, 16,
                NULL,
                PBTN_TALIGN_CENTER,
                xifmon_image.disconnect_label,
                imw,
		IMWDisconnectPBCB
            )
        )
            return(-1);
        PBtnSetHintMessage(
            &imw->disconnect_btn,
            DISCONNECT_BTN_HINT_MESG
        );

        x = 74;
        y = 8;
        if(
            PBtnInit(
                &imw->stats_btn,
                imw->toplevel,
                x, y,
                16, 16,
                NULL,
                PBTN_TALIGN_CENTER,
                xifmon_image.stats_label,
                imw,
                IMWStatsPBCB
            ) 
        )
            return(-1);
        PBtnSetHintMessage(
            &imw->stats_btn,
            STATS_BTN_HINT_MESG
        );


	x = 15;
        y = 17;
        if(
            PBtnInit(
                &imw->exit_btn,
                imw->toplevel,
                x, y,
                8, 8,
                NULL,
                PBTN_TALIGN_CENTER,
                xifmon_image.exit_label,
                imw,
                IMWExitPBCB
            )
        )
            return(-1);
        PBtnSetHintMessage(
            &imw->exit_btn,
            EXIT_BTN_HINT_MESG
        );


	/* Menu. */
	if(
	    MenuInit(
		&imw->menu,
		osw_gui[0].root_win,
                IMWMenuCB,
                imw
            )
        )
            return(-1);

        MenuAddItem(
            &imw->menu,
            "Statistics...",
            MENU_ITEM_TYPE_ENTRY,
            NULL,
            IMW_ACTION_MAP_STATS,
            -1
        );

        MenuAddItem(
            &imw->menu,
            NULL,
            MENU_ITEM_TYPE_HR,
            NULL,
            IMW_ACTION_NONE,
            -1
        );

        MenuAddItem(
            &imw->menu,
            "Connect", 
            MENU_ITEM_TYPE_ENTRY,
            NULL,
            IMW_ACTION_CONNECT,
            -1
        );

        MenuAddItem(
            &imw->menu,
            "Disconnect", 
            MENU_ITEM_TYPE_ENTRY,
            NULL,
            IMW_ACTION_DISCONNECT,     
            -1
        );

        MenuAddItem(
            &imw->menu,
            NULL,
            MENU_ITEM_TYPE_HR,
            NULL,    
            IMW_ACTION_NONE,
            -1
        );

        MenuAddItem(
            &imw->menu,
            "Exit",
            MENU_ITEM_TYPE_ENTRY,
            NULL,    
            IMW_ACTION_EXIT,
            -1
        );


	/* Initialize stats window. */
	if(SWInit(&imw->stats_win, argc, argv))
	    return(-1);



	/* Initialize interface monitoring. */
	IMWStatsInit(imw, interface);

	/* Get status of interface now. */
	IMWStatsIFStatus(imw);


	return(0);
}

/*
 *	Draw interface monitor window.
 */
void IMWDraw(ifmon_win_struct *imw, int amount)
{
	int x, y, sx, sy, i, l, sm, rm;
	int bandwidth;
	time_t dt;
	unsigned int width, height;
	pixmap_t pixmap;
	win_t w;
	image_t *src_img, *tar_img;
	win_attr_t wattr;
	double sgamma, rgamma;
	WColorStruct color;
	font_t *prev_font;

const int t_leds = 6;


	if(imw == NULL)
	    return;


	/* Map as needed. */
	if(!imw->map_state)
	{
	    OSWMapRaised(imw->toplevel);
	    imw->map_state = 1;
	    amount = IMW_DRAW_AMOUNT_COMPLETE;

            PBtnMap(&imw->connect_btn);
            PBtnMap(&imw->disconnect_btn);
            PBtnMap(&imw->exit_btn);
            PBtnMap(&imw->stats_btn);
	}

	OSWGetWindowAttributes(imw->toplevel, &wattr);

	/* Recreate buffers as needed. */
	if(imw->toplevel_buf == None)
	{
	    if(
		OSWCreatePixmap(
		    &imw->toplevel_buf,
		    wattr.width, wattr.height
		)
	    )
		return;
	}

        if(imw->toplevel_img == NULL)
        {
            if(
                OSWCreateImage(
                    &imw->toplevel_img,
                    wattr.width, wattr.height
                )
            )
                return;  
        }


	prev_font = OSWQueryCurrentFont();


	/* Draw background. */
	if(amount == IMW_DRAW_AMOUNT_COMPLETE)
	{
	    src_img = xifmon_image.bkg;
	    pixmap = imw->toplevel_buf;
	    w = imw->toplevel;

	    if(src_img != NULL)
	    {
		OSWPutImageToDrawable(src_img, pixmap);
	    }

	    OSWPutBufferToWindow(w, pixmap);
	}

	/* Draw leds. */
	if((amount == IMW_DRAW_AMOUNT_COMPLETE) ||
           (amount == IMW_DRAW_AMOUNT_LEDS)
	)
	{
	    pixmap = imw->toplevel_buf;
	    w = imw->toplevel;
	    sx = 96;
	    sy = 10;

	    width = 100;
	    height = 18;

            src_img = xifmon_image.bkg;
	    tar_img = imw->toplevel_img;

	    /* Copy segment of background. */
	    if((src_img != NULL) &&
               (tar_img != NULL)
	    )
	        BlitBufAbsolute(
		    osw_gui[0].depth,
		    (u_int8_t *)tar_img->data,
		    (u_int8_t *)src_img->data,
		    sx, sy,
		    tar_img->width, tar_img->height,
		    sx, sy,
                    src_img->width, src_img->height,
                    width, height,
		    1, 1
		);


	    /* Leds. */
	    bandwidth = imw->bandwidth / 8;	/* Get bandwidth and convert
						 * to bytes per second.
						 */
	    dt = imw->cur_time - imw->last_time;	/* Delta time. */
	    if((bandwidth > 0) && (dt > 0))
	        sgamma = ((double)imw->sx_delta / (double)bandwidth) *
		    (1000.0 / (double)dt);
	    else
		sgamma = 0.0;
            if((bandwidth > 0) && (dt > 0))
                rgamma = ((double)imw->rx_delta / (double)bandwidth) *
                    (1000.0 / (double)dt);
            else
                rgamma = 0.0;

	    /* Gamma values are now in range of [0, 1]. */

	    sgamma = sgamma * (double)t_leds;
	    rgamma = rgamma * (double)t_leds;

	    if(sgamma > 0)
	        sm = (int)(sgamma + 1);
	    else
		sm = 0;
	    if(rgamma > 0)
	        rm = (int)(rgamma + 1);
	    else
                rm = 0;

	    if(sm > t_leds)
	    {
		sm = t_leds;
		sgamma = sm;
	    }
            if(rm > t_leds)
            {
                rm = t_leds;
                rgamma = rm;
            }

	    sgamma = sgamma - (double)(sm - 1);
            rgamma = rgamma - (double)(rm - 1);

	    /* Send leds. */
            src_img = xifmon_image.led_cursor;
            x = 96;	/* Start position is slightly offsetted from sx and sy. */
            y = 11;

            if((src_img != NULL) &&
               (tar_img != NULL)
	    )
	    {
		l = sm - 1;	/* Last on led. */

		/* Greens. */
		for(i = 0; i < (t_leds - 2); i++)
		{
		    if(i >= sm)
			break;

                    if(i == l)
                    {
                        color.a = 0x00;
                        color.r = 0x00;
                        color.g = (u_int8_t)(0xff * sgamma);
                        color.b = 0x00;
                    }
                    else
                    {
                        color.a = 0x00;
                        color.r = 0x00;
                        color.g = 0xff;
                        color.b = 0x00;
                    }

		    BlitBufCursor(
                        osw_gui[0].depth,
                        (u_int8_t *)tar_img->data,
                        (u_int8_t *)src_img->data,
                        x, y,
                        tar_img->width, tar_img->height,
                        src_img->width, src_img->height,
                        color
                    );

		    x += src_img->width;
		}

                /* Yellows. */
                for(; i < (t_leds - 1); i++)
                {
                    if(i >= sm)
                        break;

                    if(i == l)
                    {
                        color.a = 0x00;
                        color.r = (u_int8_t)(0xff * sgamma);
                        color.g = (u_int8_t)(0xff * sgamma);
                        color.b = 0x00;
                    }
                    else
                    {
                        color.a = 0x00;
                        color.r = 0xff;
                        color.g = 0xff;
                        color.b = 0x00;
                    }

                    BlitBufCursor(
                        osw_gui[0].depth,
                        (u_int8_t *)tar_img->data,   
                        (u_int8_t *)src_img->data,   
                        x, y,
                        tar_img->width, tar_img->height,
                        src_img->width, src_img->height,
                        color
                    );
         
                    x += src_img->width;
                }

                /* Reds. */
                for(; i < t_leds; i++)
                {
                    if(i >= sm)
                        break;

                    if(i == l)
                    {
                        color.a = 0x00;
                        color.r = (u_int8_t)(0xff * sgamma);
                        color.g = 0x00;
                        color.b = 0x00;
                    }
                    else
                    {
                        color.a = 0x00;
                        color.r = 0xff;
                        color.g = 0x00;
                        color.b = 0x00;
                    }

                    BlitBufCursor(
                        osw_gui[0].depth,
                        (u_int8_t *)tar_img->data,
                        (u_int8_t *)src_img->data,
                        x, y,
                        tar_img->width, tar_img->height,
                        src_img->width, src_img->height,
                        color
                    );

                    x += src_img->width;
                }
	    }

            /* Recv leds. */
            x = 96;     /* Start position is slightly offsetted from sx and sy. */
            y = 18;

            if((src_img != NULL) &&
               (tar_img != NULL)
            )        
            {
                l = rm - 1;	/* Last on led. */

		/* Greens. */
                for(i = 0; i < (t_leds - 2); i++)
                {
                    if(i >= rm)
                        break;

                    if(i == l)
                    {
                        color.a = 0x00;
                        color.r = 0x00;
                        color.g = (u_int8_t)(0xff * rgamma);
                        color.b = 0x00;
                    }   
                    else
                    {
                        color.a = 0x00;
                        color.r = 0x00;
                        color.g = 0xff;
                        color.b = 0x00;
                    }

                    BlitBufCursor(
                        osw_gui[0].depth,
                        (u_int8_t *)tar_img->data,
                        (u_int8_t *)src_img->data,
                        x, y,
                        tar_img->width, tar_img->height,
                        src_img->width, src_img->height,
                        color
                    );

                    x += src_img->width;
                }

                /* Yellows. */
                for(; i < (t_leds - 1); i++)
                {
                    if(i >= rm)
                        break;

                    if(i == l)
                    {
                        color.a = 0x00;
                        color.r = (u_int8_t)(0xff * rgamma);
                        color.g = (u_int8_t)(0xff * rgamma);
                        color.b = 0x00;
                    }
                    else
                    {   
                        color.a = 0x00;
                        color.r = 0xff;
                        color.g = 0xff;
                        color.b = 0x00;
                    }
                     
                    BlitBufCursor(
                        osw_gui[0].depth,
                        (u_int8_t *)tar_img->data, 
                        (u_int8_t *)src_img->data, 
                        x, y,
                        tar_img->width, tar_img->height,
                        src_img->width, src_img->height,
                        color
                    );

                    x += src_img->width;
                }

		/* Reds. */
                for(; i < t_leds; i++)
                {
                    if(i >= rm)
                        break;

                    if(i == l)
                    {
                        color.a = 0x00;
                        color.r = (u_int8_t)(0xff * rgamma);
                        color.g = 0x00;
                        color.b = 0x00;
                    }
                    else
                    {   
                        color.a = 0x00;
                        color.r = 0xff;
                        color.g = 0x00;
                        color.b = 0x00;
                    }

                    BlitBufCursor(
                        osw_gui[0].depth,
                        (u_int8_t *)tar_img->data,
                        (u_int8_t *)src_img->data,
                        x, y,
                        tar_img->width, tar_img->height,
                        src_img->width, src_img->height,
                        color
                    );

                    x += src_img->width;
                }

	    }

	    /* Put segment of image to window. */
	    OSWPutImageToDrawableSect(
		tar_img, w,
		sx, sy,
		sx, sy,
		width, height
	    );
	}


	/* IF status lights. */
        if((amount == IMW_DRAW_AMOUNT_COMPLETE) ||
           (amount == IMW_DRAW_AMOUNT_IFSTATE)
        )
	{
            pixmap = imw->toplevel_buf;
            w = imw->toplevel;
            sx = 0;
            sy = 14;

            width = 10;
            height = 14;

            src_img = xifmon_image.bkg;
            tar_img = imw->toplevel_img;


            /* Copy segment of background. */
            if((src_img != NULL) &&
               (tar_img != NULL)
            )
                BlitBufAbsolute(
                    osw_gui[0].depth,
                    (u_int8_t *)tar_img->data,
                    (u_int8_t *)src_img->data,
                    sx, sy,
                    tar_img->width, tar_img->height,
                    sx, sy,
                    src_img->width, src_img->height,
                    width, height,
                    1, 1
                );    

	    /* Draw if status lights. */
	    src_img = xifmon_image.if_light_cursor;
	    if((src_img != NULL) &&
               (tar_img != NULL)
            )
	    {
	        switch(imw->if_state)
	        {
		  case IF_STATE_ON:
		    x = 2;
		    y = 14;

                    color.a = 0x00;
                    color.r = 0x00;
                    color.g = 0xff;
                    color.b = 0x00;

                    BlitBufCursor(
                        osw_gui[0].depth,
                        (u_int8_t *)tar_img->data,   
                        (u_int8_t *)src_img->data,
                        x, y,
                        tar_img->width, tar_img->height,
                        src_img->width, src_img->height,
                        color
                    );
		    break;


		  case IF_STATE_STARTUP:
                    x = 0;
                    y = 21;

                    color.a = 0x00;
                    color.r = 0xff;
                    color.g = 0x80;
                    color.b = 0x00;

                    BlitBufCursor(
                        osw_gui[0].depth,
                        (u_int8_t *)tar_img->data,
                        (u_int8_t *)src_img->data,
                        x, y,
                        tar_img->width, tar_img->height,
                        src_img->width, src_img->height,
                        color
                    );
		    break;

		  default:
		    break;
		}

                /* Put segment of image to window. */
                OSWPutImageToDrawableSect(
                    tar_img, w,   
                    sx, sy,
                    sx, sy,
                    width, height
                );
	    }
	}


	OSWSetFont(prev_font);


	return;
}


int IMWManage(ifmon_win_struct *imw, event_t *event)
{
	int x, y, root_x, root_y;
	win_attr_t wattr;
	static int last_x, last_y;
	int events_handled = 0;


        if((imw == NULL) ||
           (event == NULL)
	)
            return(events_handled);

	if(!imw->map_state &&
	   (event->type != MapNotify)
	)
	    return(events_handled);


	switch(event->type)
	{
	  case ButtonPress:
            if(event->xany.window == imw->toplevel)
            {
		switch(event->xbutton.button)
		{
		  case Button1:
		    /* Move window. */
		    OSWGetPointerCoords(
			imw->toplevel,
			&root_x, &root_y,
			&x, &y
		    );

		    last_x = x;
		    last_y = y;

		    imw->in_drag = True;
		    break;

                  case Button3:
		    /* Map menu. */
                    OSWGetPointerCoords(
                        imw->toplevel,
                        &root_x, &root_y,
                        &x, &y
                    );

                    if(imw->menu.map_state)
                        MenuUnmap(&imw->menu);
                    else
                        MenuMapPos(
                            &imw->menu,
                            root_x,
                            root_y
                        );
		    break;
		}
		events_handled++;
                return(events_handled);
            }
	    break;

	  case ButtonRelease:
            if(event->xany.window == imw->toplevel)
            {
                imw->in_drag = False;

                events_handled++;
                return(events_handled);
            }
            break;

	  case MotionNotify:
	    if(event->xany.window == imw->toplevel)
            {
		if(imw->in_drag)
		{
                    OSWGetPointerCoords(
                        imw->toplevel,   
                        &root_x, &root_y,
                        &x, &y
                    );

		    OSWGetWindowAttributes(imw->toplevel, &wattr);
		    OSWMoveWindow(
			imw->toplevel,
			root_x - last_x,
			root_y - last_y
		    );
		}
                events_handled++;
                return(events_handled);
            }
            break;

	  case Expose:
	    if(event->xany.window == imw->toplevel)
	    {
		IMWDraw(imw, IMW_DRAW_AMOUNT_COMPLETE);

                events_handled++;
                return(events_handled);
	    }
	    break;

	}


	/* Manage push buttons. */
	if(events_handled == 0)
            events_handled += PBtnManage(&imw->connect_btn, event);

        if(events_handled == 0)
            events_handled += PBtnManage(&imw->disconnect_btn, event);

        if(events_handled == 0)
            events_handled += PBtnManage(&imw->exit_btn, event);

        if(events_handled == 0)
            events_handled += PBtnManage(&imw->stats_btn, event);

	/* Manage menu. */
	if(events_handled == 0)
            events_handled += MenuManage(&imw->menu, event);


	/* Manage stats window. */
        if(events_handled == 0)
            events_handled += SWManage(&imw->stats_win, event);


	return(events_handled);
}


void IMWMap(ifmon_win_struct *imw)
{
        if(imw == NULL)
            return;

	imw->map_state = 0;
	IMWDraw(imw, IMW_DRAW_AMOUNT_COMPLETE);
}


void IMWUnmap(ifmon_win_struct *imw)
{
        if(imw == NULL)
            return;


	MenuUnmap(&imw->menu);

        PBtnUnmap(&imw->stats_btn);
        PBtnUnmap(&imw->connect_btn);
        PBtnUnmap(&imw->disconnect_btn);
        PBtnUnmap(&imw->exit_btn);

        OSWUnmapWindow(imw->toplevel);
	imw->map_state = 0;


        OSWDestroyImage(&imw->toplevel_img);
        OSWDestroyPixmap(&imw->toplevel_buf);
}


void IMWDestroy(ifmon_win_struct *imw)
{
        if(imw == NULL)
            return;


	IMWStatsShutdown(imw);


	/* Stats window. */
	SWDestroy(&imw->stats_win);


	if(IDC())
	{
            MenuDestroy(&imw->menu);

	    PBtnDestroy(&imw->stats_btn);
	    PBtnDestroy(&imw->connect_btn);
            PBtnDestroy(&imw->disconnect_btn);
            PBtnDestroy(&imw->exit_btn);

	    OSWDestroyWindow(&imw->toplevel);
	    OSWDestroyImage(&imw->toplevel_img);
	    OSWDestroyPixmap(&imw->toplevel_buf);
	}

	free(imw->connect_script);
	imw->connect_script = NULL;

	free(imw->disconnect_script);
        imw->disconnect_script = NULL;

        free(imw->interface);
	imw->interface = NULL;

	imw->map_state = 0;
	imw->is_in_focus = 0;
	imw->x = 0;
	imw->y = 0;
	imw->width = 0;
	imw->height = 0;

        imw->in_drag = False;

        imw->if_state = IF_STATE_OFF;


	return;
}
