/*
                    Interface Monitor Window Stats IO

	Functions:

	int IMWStatsInit(ifmon_win_struct *imw, const char *interface)
	int IMWStatsFetch(ifmon_win_struct *imw)
	int IMWStatsIFStatus(ifmon_win_struct *imw)
	void IMWStatsShutdown(ifmon_win_struct *imw)

	---

 */

#include <stdio.h>
#include <stdlib.h>
#include "ifstats.h"
#include "../include/osw-x.h"
#include "../include/widget.h"
#include "xifmon.h"


/*
 *	Open connection to interface and store if stats descriptor
 *	information on imw.
 */
int IMWStatsInit(ifmon_win_struct *imw, const char *interface)
{
	int status;
	if_stats_desc_struct *ifsd;


	if(imw == NULL)
	    return(-1);

	imw->if_data = NULL;


	/* Allocate new interface stats structure. */
	ifsd = (if_stats_desc_struct *)calloc(
	    1, sizeof(if_stats_desc_struct)
	);
	if(ifsd == NULL)
            return(-1);

	/* Attempt to initialize and connect to interface. */
	status = IFSInit(ifsd, interface);
	if(status)
	{
	    free(ifsd);
	    return(-1);
	}


	/* Successfully initialized interface. */
	imw->if_data = ifsd;


	return(0);
}

/*
 *	Updates basic interface stats on imw from interface.
 */
int IMWStatsFetch(ifmon_win_struct *imw)
{
	int status;
	if_stats_desc_struct *ifsd;
	stats_win_struct *sw;

	int ptval;
	u_int8_t *gp_ptr1, *gp_ptr2;
	int bandwidth;
	time_t dt;


	if(imw == NULL)
	    return(-1);

	if(imw->if_data == NULL)
	    return(-1);


	/* Get pointers to structures. */
	ifsd = (if_stats_desc_struct *)imw->if_data;
	sw = &imw->stats_win;


	/* ******************************************************** */

	/* Fetch stats from interface. */
	status = IFSFetch(ifsd);
	if(status)
	    return(-1);

	/* Put values into imw structure. */
	if(ifsd->rx_bytes > imw->rx_total)
	    imw->rx_delta = ifsd->rx_bytes - imw->rx_total;
	else
	    imw->rx_delta = 0;
	imw->rx_total = ifsd->rx_bytes;

        if(ifsd->sx_bytes > imw->sx_total)
            imw->sx_delta = ifsd->sx_bytes - imw->sx_total;
        else
            imw->sx_delta = 0;
        imw->sx_total = ifsd->sx_bytes;


	/* Update time bounds. */
	if(imw->cur_time > cur_millitime)
	{
	    /* Time looped, so reset. */
            imw->last_time = 0;
            imw->cur_time = cur_millitime;
	}
	else
	{
	    imw->last_time = imw->cur_time;
	    imw->cur_time = cur_millitime;
	}

	/* Redraw imw. */
	if(imw->map_state)
	    IMWDraw(imw, IMW_DRAW_AMOUNT_LEDS);

	/* Put values into sw prompts (if mapped). */
	if(sw->map_state)
	{
	    PromptSetUL(&sw->total_rx_prompt, ifsd->rx_bytes);
	    PromptSetUL(&sw->total_sx_prompt, ifsd->sx_bytes);

            PromptSetUL(&sw->errors_rx_prompt, ifsd->rex_bytes);
            PromptSetUL(&sw->errors_sx_prompt, ifsd->sex_bytes);

            PromptSetUL(&sw->packets_rx_prompt, ifsd->rx_packets);
            PromptSetUL(&sw->packets_sx_prompt, ifsd->sx_packets);

            PromptSetUL(&sw->lrqs_rx_prompt, ifsd->rx_lrqs);
            PromptSetUL(&sw->lrqs_sx_prompt, ifsd->sx_lrqs);
	}

        /* ******************************************************** */

	/* Time to update status window? */
	if((cur_millitime - sw->last_update) >=
           (20 * 1000 / 150)
	)
	{
	    bandwidth = imw->bandwidth / 8;	/* Get bandwidth and convert to
						 * bytes per second.
						 */
	    dt = imw->cur_time - imw->last_time;        /* Delta time. */

	    /* Recieve graph points. */
	    if((sw->total_graph_pts_rx > 0) &&
               (bandwidth > 0) &&
               (dt > 0) &&
               (sw->graph_pt_rx != NULL)
	    )
	    {
		/* Calculate value of new graph point. */
	        ptval = 255 * imw->rx_delta / (unsigned long)bandwidth *
		    1000 / (unsigned long)dt;
		if(ptval > 255)
		    ptval = 255;
		else if(ptval < 0)
		    ptval = 0;

		/* Shift graph points. */
		for(
		    gp_ptr1 = &sw->graph_pt_rx[sw->total_graph_pts_rx - 1],
                    gp_ptr2 = &sw->graph_pt_rx[sw->total_graph_pts_rx - 2];
		    gp_ptr2 >= sw->graph_pt_rx;
		    gp_ptr1--, gp_ptr2--
		)
		    *gp_ptr1 = *gp_ptr2;

		*sw->graph_pt_rx = (u_int8_t)ptval;
	    }

            /* Send graph points. */
            if((sw->total_graph_pts_sx > 0) &&
               (bandwidth > 0) &&
               (dt > 0) &&
               (sw->graph_pt_sx != NULL)
            )
            {
                /* Calculate value of new graph point. */
                ptval = 255 * imw->sx_delta / (unsigned long)bandwidth *
                    1000 / (unsigned long)dt;
                if(ptval > 255)
                    ptval = 255;
                else if(ptval < 0)
                    ptval = 0;

                /* Shift graph points. */
                for(
                    gp_ptr1 = &sw->graph_pt_sx[sw->total_graph_pts_sx - 1],
                    gp_ptr2 = &sw->graph_pt_sx[sw->total_graph_pts_sx - 2];
                    gp_ptr2 >= sw->graph_pt_sx;
                    gp_ptr1--, gp_ptr2--
                )
                    *gp_ptr1 = *gp_ptr2;

                *sw->graph_pt_sx = (u_int8_t)ptval;
            }


            /* Redraw sw. */
            if(sw->map_state)
                SWDraw(sw, SW_DRAW_AMOUNT_GRAPH);

	    sw->last_update = cur_millitime;
	}


	return(0);
}

/*
 *	Checks status of interface and updates status of interface
 *	on imw.
 */
int IMWStatsIFStatus(ifmon_win_struct *imw)
{
	if_stats_desc_struct *ifsd;
	stats_win_struct *sw;
	char text[256];


	if(imw == NULL)
	    return(-1);

        if(imw->if_data == NULL)
            return(-1);

        ifsd = (if_stats_desc_struct *)imw->if_data;
 

	/* Check current state of interface recorded on imw. */
	switch(imw->if_state)
	{
	  case IF_STATE_ON:
	    /* Interface was up, check if it's still up. */
	    if(!IFSIsUp(ifsd))
	    {
		imw->if_state = IF_STATE_OFF;

                if(imw->map_state)
		    IMWDraw(imw, IMW_DRAW_AMOUNT_IFSTATE);
	    }
	    break;

	  case IF_STATE_STARTUP:
	    /* Was trying to connect to interface. */
            if(IFSIsUp(ifsd))
            {
                imw->if_state = IF_STATE_ON;
		IFSGetIP(ifsd);

                if(imw->map_state)
                    IMWDraw(imw, IMW_DRAW_AMOUNT_IFSTATE);
            }
            break;

	  default:
	    /* Interface was off, check if its now on. */
            if(IFSIsUp(ifsd))
            {
                imw->if_state = IF_STATE_ON;
		IFSGetIP(ifsd);

                if(imw->map_state)
                    IMWDraw(imw, IMW_DRAW_AMOUNT_IFSTATE);
            }
	    break;
	}


	/* If up get additional information about status. */
	if(imw->if_state == IF_STATE_ON)
	{
            /* Interface state was last checked to be DOWN. */
	    int displayed_bandwidth;


	    /* Get pointer to stats window. */
	    sw = &imw->stats_win;

	    /* Get IP address. */
	    IFSGetIP(ifsd);
	    sprintf(text, "%i.%i.%i.%i",
		ifsd->ip.charaddr[0],
                ifsd->ip.charaddr[1],
                ifsd->ip.charaddr[2],
                ifsd->ip.charaddr[3]
	    );
	    PromptSetS(&sw->ip_prompt, text);

	    /* Get bandwidth. */
	    IFSGetBandwidth(ifsd);
/* Use IMW's bandwidth value for now. ifsd's bandwidth isn't
 * correct.
 */
	    switch(imw->bandwidth_units)
	    {
	      case IF_BANDWIDTH_UNITS_BITS_PER_SECOND:
		displayed_bandwidth = imw->bandwidth;
		break;
	      default:	/* IF_BANDWIDTH_UNITS_BYTES_PER_SECOND */
		displayed_bandwidth = imw->bandwidth / 8;
		break;
	    }
	    PromptSetI(&sw->bandwidth_prompt, displayed_bandwidth);
	}
	else
	{
	    /* Interface state was last checked to be DOWN. */

            /* Get pointer to stats window. */
            sw = &imw->stats_win;

	    /* Set IP address to 0.0.0.0 */
            PromptSetS(&sw->ip_prompt, "0.0.0.0");

/* Leave bandwidth alone. */

	}

	return(0);
}

/*
 *	Closes referances to interface and deallocates
 *	resources in the if_data structure on imw.
 */
void IMWStatsShutdown(ifmon_win_struct *imw)
{
        if_stats_desc_struct *ifsd;


        if(imw == NULL)
            return;

        if(imw->if_data == NULL)
            return;

	/* Shut down interface stats descriptor. */        
        ifsd = (if_stats_desc_struct *)imw->if_data;
	if(ifsd != NULL)
	{
	    /* Shut down connection to interface. */
	    IFSShutdown(ifsd);

	    /* Free interface data structure that we allocated. */
	    free(ifsd);
	    imw->if_data = ifsd = NULL;
	}
}
