/*
                         PPP Stats Demo Source

	This is a sample program showing out to get PPP statistics
	from the ppp0 interface for Linux kernels 2.2.x.


 */

#ifndef __linux__
#warning Target system does not appear to be Linux.
#warning This program must be compiled for Linux as the target system.
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/time.h>
#include <signal.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netdb.h>
#include <unistd.h>

#include <linux/in.h>
#include <linux/if.h>
#include <linux/ppp_defs.h>
#include <linux/if_ppp.h>

#ifndef SHUT_RDWR
# define SHUT_RDWR	2
#endif

/* For IP addresses. */
typedef union {
	u_int32_t whole;
	u_int8_t charaddr[4];
} ip_addr_union;


int runlevel;

void SigHandler(int s);
int OpenService(char *service, char *protocol);
void GetMilliTime(long *t);
u_int32_t GetInterfaceIP(char *ifname);


/*
 *	Handles signals.
 */
void SigHandler(int s)
{
	switch(s)
	{
	  case SIGINT:
	    runlevel = 1;
	    break;

	  case SIGTERM:
	    runlevel = 1;
	    break;

	  case SIGSEGV:
	    fprintf(stderr, "Segmentation fault.\n");
            runlevel = 1;
	    break;

          case SIGPIPE:
            signal(SIGPIPE, SigHandler);
            break;
	}

	return;
}


/*
 *	Creates a listening socket and binds it according to the
 *	information specified by the given service.
 */
int OpenService(char *service, char *protocol)
{
        struct servent *sp;
        struct sockaddr_in addr[1];
        int s, p;


	sp = getservbyname(service, protocol);
        if(sp == NULL)
        {
            perror("getservbyname");
            return(-1);
        }
        else
        {
            p = sp->s_port;
        }

	s = socket(AF_INET, SOCK_STREAM, 0);
	if(s < 0)
        {
            perror("socket");
            return(-1);
        }

        addr->sin_family = AF_INET;
        memset(&addr->sin_addr, 0, sizeof(addr->sin_addr));
        addr->sin_port = p;


        /* Bind the socket. */
        if(bind(s, (struct sockaddr *)addr, sizeof(addr)))
        {
            perror("bind");
            shutdown(s, SHUT_RDWR);
            return(-1);
        }

        if(listen(s, 5) < 0)
        {
            perror("listen");
            shutdown(s, SHUT_RDWR);
	    return(-1);
        }


	return(s);
}

/*
 *	Gets time since midnight in milliseconds.
 */
void GetMilliTime(long *t)
{
	struct timeval tv[1];

	if(gettimeofday(tv, NULL) < 0)
	    return;

	*t = (tv->tv_sec % 86400) * 1000 + tv->tv_usec / 1000; 

	return;
}


/*
 *	Fetches the IP of the specified interface in network byte
 *	order.
 */
u_int32_t GetInterfaceIP(char *ifname)
{
        int s;
        struct ifreq ifr;
        struct in_addr ina;
        u_int32_t ip = 0x00000000;


	s = socket(AF_INET, SOCK_DGRAM, 0);
	if(s < 0)
	{
	    perror("socket");
	    return(ip);
        }

	/*   Note that ifr_name is #defined to
	 *   ifr_ifrn.ifrn_name. (See linux/if.h).
	 */
        strncpy(
	    ifr.ifr_name,
	    ifname,
	    IFNAMSIZ
	);
	ifr.ifr_name[IFNAMSIZ - 1] = '\0';


        /* Check if interface device is up first. */
        if(ioctl(s, SIOCGIFADDR, &ifr) < 0)
        {
            close(s);
            return(ip);
        }

        /* Give struct in_addr ina the address of the device. */
        ina = ((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr;
        ip = ina.s_addr;	/* It's in network byte order. */


	/* Close socket. */
        close(s);


        return(ip);
}


int main(int argc, char *argv[])
{
	char *service = "ifmon";
	char *protocol = "tcp";
	char *ifname = "ppp0";

	long	cur_time = 0,
		prev_time = 0;

	int s;
	struct ifreq ifr;
	struct ppp_stats ppp_info[1];
	struct pppstat	*ppp_st_ptr,
			last_ppp_st[1];

	unsigned long	disgarded_frames = 0,
			rx_bytes = 0,
			sx_bytes = 0,
			rex_bytes = 0,
			sex_bytes = 0,
			rx_packets = 0,
			sx_packets = 0,
			rx_lrqs = 0,
			sx_lrqs = 0;
	ip_addr_union	ip;


	/* Switch runlevel to 1. */
	runlevel = 1;

	/* Watch for these signals. */
	signal(SIGINT, SigHandler);
	signal(SIGTERM, SigHandler);
        signal(SIGSEGV, SigHandler);
        signal(SIGPIPE, SigHandler);


	/* Open service, set up listening port and socket. */
	s = OpenService(service, protocol);
	if(s < 0)
	{
	    fprintf(
		stderr,
"Could not set up listening socket.\n\
Make sure that you have the service %s defined in /etc/services\n\
and that no other server programs are bind()ed to its port number.\n",
		service
	    );
	    return(1);
	}

	/* Clear interface request struct. */
	memset(&ifr, 0, sizeof(struct ifreq));


        /* More PPP interface listening setups... */
	strncpy(
	    ifr.ifr_ifrn.ifrn_name,
	    ifname,
	    IFNAMSIZ
	);
	ifr.ifr_ifrn.ifrn_name[IFNAMSIZ - 1] = '\0';

        ifr.ifr_ifru.ifru_data = (caddr_t)ppp_info;
        ppp_st_ptr = &ppp_info->p;
        memset(last_ppp_st, 0, sizeof(last_ppp_st));



	/* Update timming before we start the main while() loop. */
	GetMilliTime(&cur_time);
	prev_time = cur_time;

        /* Begin fetching stats. */
	ip.whole = GetInterfaceIP(ifname);
	if(ip.whole != 0x00000000)
            fprintf(  
                stdout,
 "Monitoring interface %s on %i.%i.%i.%i (press ctrl+c to interupt)...\n",
		ifname,
		ip.charaddr[0],
                ip.charaddr[1],
                ip.charaddr[2],
                ip.charaddr[3]
	    );
	else
            fprintf(
                stdout,
 "Cannot find interface %s (Press ctrl+c to interupt)...\n",
	        ifname
	    );

	runlevel = 2;
	while(runlevel >= 2)
	{
	    /* Fetch stats. */  
            ioctl(s, SIOCGPPPSTATS, (caddr_t)&ifr);


	    /* Total disgarded frames. */
	    disgarded_frames = ppp_st_ptr->ppp_discards;

	    /* Total recieved and sent bytes. */
	    rx_bytes = ppp_st_ptr->ppp_ibytes;
            sx_bytes = ppp_st_ptr->ppp_obytes;

	    /* Total errors. */
            rex_bytes = ppp_st_ptr->ppp_ierrors;
	    sex_bytes = ppp_st_ptr->ppp_oerrors;

	    /* Total recieved and sent packets. */
            rx_packets = ppp_st_ptr->ppp_ipackets;
            sx_packets = ppp_st_ptr->ppp_opackets;

	    /* Total LRQ frames recieved and sent. */
	    rx_lrqs = ppp_st_ptr->ppp_ilqrs;
	    sx_lrqs = ppp_st_ptr->ppp_olqrs;


	    /* Print the stats. */
	    fprintf(
		stdout,
		"\r RX: %ld bytes  %ld packets  SX: %ld bytes  %ld packets ",
		rx_bytes, rx_packets,
		sx_bytes, sx_packets
	    );
	    fflush(stdout);

	    usleep(1000000);
	}


	/* Begin shutting down, close listening socket. */
	shutdown(s, SHUT_RDWR);

	fprintf(
	    stdout,
	    "\nClosed connection to interface %s.\n",
	    ifname
	);


	return(0);
}
